@@ -1,8 +1,8 @@
 import logging
-from pprint import pformat
 from typing import List, Type
 
 from sqlalchemy import create_engine
+from sqlalchemy import exc
 from sqlalchemy.ext.declarative import declarative_base
 from sqlalchemy.orm import sessionmaker, Session, joinedload
 
@@ -56,7 +56,13 @@ class SQADataSource(DataSource):
 
     def delete(self, entity) -> bool:
         self.session.delete(entity)
-        return self.session.commit()
+        try:
+            self.session.commit()
+            return True
+        except exc.SQLAlchemyError as err:
+            logging.exception(msg="Cannot delete", exc_info=err)
+            self.session.rollback()
+            return False
 
     def all(self, of_type, limit=None) -> set:
         return self.session.query(of_type).limit(limit).all()
@@ -91,7 +97,8 @@ class SQADataSource(DataSource):
         try:
             self.session.commit()
             return True
-        except:
+        except exc.SQLAlchemyError as err:
+            logging.exception(msg="Cannot persist", exc_info=err)
             self.session.rollback()
             return None
 
