@@ -15,6 +15,7 @@ from sync.operations import (
     SkipUpdateOrphanedEntity,
     DeletedOrphanedEntity,
     CouldNotCreate,
+    CouldNotDelete,
 )
 
 MODES_TO_PARAMETERS = {
@@ -59,8 +60,42 @@ class Syncer:
             operations.extend(
                 self._sync_from_to(self.right_ds, self.left_ds, mapping.swapped())
             )
+
         return operations
 
+    def _collect_linked_classes(self, entity_class):
+        result = set()
+        for mapping in self.mappings:
+            if entity_class in mapping.entity_types:
+                index = mapping.entity_types.index(entity_class)
+                other_class = mapping.entity_types[1 - index]
+                # check if class may be created by other_class
+                if (index == 0 and Mode.RIGHT_TO_LEFT in mapping.modes) or (
+                    index == 1 and Mode.LEFT_TO_RIGHT in mapping.modes
+                ):
+                    result.add(other_class)
+        return result
+
+    def _is_orphaned(self, generating_classes, entity, entity_ds, other_ds):
+        for generating_class in generating_classes:
+            others = self.link_provider.others(
+                entity, entity_ds, generating_class, other_ds
+            )
+            if others:
+                for other in others:
+                    refreshed_other = other_ds.refresh(other)
+                    if not refreshed_other:
+                        return True
+        return False
+
+    def _collect_orphaned_entities(self, entity_ds, entity_class, other_ds):
+        generating_classes = self._collect_linked_classes(entity_class)
+        return [
+            entity
+            for entity in entity_ds.all(entity_class)
+            if self._is_orphaned(generating_classes, entity, entity_ds, other_ds)
+        ]
+
     def _preload_data_sources(self):
         def preload_data_source_type(attribute, data_source, hint, index, key, mapping):
             path = getattr(key, attribute)
@@ -95,7 +130,10 @@ class Syncer:
         operations = []
         try:
             conditions = self._create_key_value_pairs(
-                from_entity, mapping.keys, from_ds=from_ds, to_ds=data_source,
+                from_entity,
+                mapping.keys,
+                from_ds=from_ds,
+                to_ds=data_source,
             )
             found_entities = data_source.find(entity_type, conditions)
         except NotLinked as e:
@@ -110,7 +148,10 @@ class Syncer:
 
             try:
                 non_key_attributes = self._create_key_value_pairs(
-                    from_entity, mapping.attributes, from_ds=from_ds, to_ds=data_source,
+                    from_entity,
+                    mapping.attributes,
+                    from_ds=from_ds,
+                    to_ds=data_source,
                 )
                 to_entity = Syncer._create_entity(
                     conditions, non_key_attributes, entity_type, from_entity_timestamp
@@ -244,10 +285,16 @@ class Syncer:
         try:
             attributes = {
                 **self._create_key_value_pairs(
-                    from_entity, mapping.keys, from_ds=from_ds, to_ds=to_ds,
+                    from_entity,
+                    mapping.keys,
+                    from_ds=from_ds,
+                    to_ds=to_ds,
                 ),
                 **self._create_key_value_pairs(
-                    from_entity, mapping.attributes, from_ds=from_ds, to_ds=to_ds,
+                    from_entity,
+                    mapping.attributes,
+                    from_ds=from_ds,
+                    to_ds=to_ds,
                 ),
             }
         except NotLinked as e:
@@ -271,16 +318,6 @@ class Syncer:
                     )
                 )
 
-                if mapping.delete_orphaned_entities:
-                    if from_ds.delete(from_entity):
-                        self.link_provider.unlink(
-                            from_entity, entity_ds=from_ds, other_ds=to_ds
-                        )
-                        operations.append(
-                            DeletedOrphanedEntity(
-                                from_entity.__class__, to_class, from_entity, to_entity
-                            )
-                        )
                 continue
             else:
                 to_entity = refreshed_to_entity
@@ -335,7 +372,10 @@ class Syncer:
         return operations
 
     def _sync_from_to(
-        self, from_ds: DataSource, to_ds: DataSource, mapping: Mapping,
+        self,
+        from_ds: DataSource,
+        to_ds: DataSource,
+        mapping: Mapping,
     ):
         operations = []
         from_class, to_class = mapping.entity_types
@@ -346,6 +386,25 @@ class Syncer:
                 self._sync_entity(from_ds, from_entity, to_ds, to_class, mapping)
             )
 
+        if mapping.delete_orphaned_entities:
+            operations.extend(self._delete_orphaned_entities(from_ds, to_class, to_ds))
+
+        return operations
+
+    def _delete_orphaned_entities(self, from_ds, to_class, to_ds):
+        """
+        Deletes orphaned entities in `to_ds`
+        """
+        operations = []
+        for orphaned_entity in self._collect_orphaned_entities(
+            to_ds, to_class, from_ds
+        ):
+            successful_deletion = to_ds.delete(orphaned_entity)
+            if not successful_deletion:
+                operations.append(CouldNotDelete(orphaned_entity))
+            else:
+                self.link_provider.unlink(orphaned_entity, to_ds, from_ds)
+                operations.append(DeletedOrphanedEntity(orphaned_entity))
         return operations
 
     def sync_entity(self, type, id):
@@ -395,7 +454,12 @@ class Syncer:
         else:
             operations.extend(
                 self._update_linked_entity(
-                    from_ds, from_entity, others, mapping, to_class, to_ds,
+                    from_ds,
+                    from_entity,
+                    others,
+                    mapping,
+                    to_class,
+                    to_ds,
                 )
             )
         return operations
